local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Utilities = import('/lua/utilities.lua')
local AIUtils = import('/lua/AI/aiutilities.lua')

local TotalTargets = 4 #the total number of targets
local TotalWaypoints = 14 #really is 14, 6 for test  #Map maker needs to input the total number of waypoints excluding WAYPOINT_0





local oldOnPopulate = OnPopulate

   function OnPopulate()
        #oldOnPopulate()
        ScenarioUtils.InitializeArmies()
	Survival_InitGame()
 	ScenarioFramework.SetPlayableArea('AREA_1', false)
   end

   local oldOnStart = OnStart
   function OnStart(self)
      oldOnStart(self)
   end


-- econ adjust based on who is playing
-- taken from original survival/Jotto
--------------------------------------------------------------------------
function ScenarioUtils.CreateResources()
	local Markers = ScenarioUtils.GetMarkers();

	for i, tblData in pairs(Markers) do -- loop marker list
		local SpawnThisResource = false; -- default to no
		if (tblData.resource and not tblData.SpawnWithArmy) then -- if this is a regular resource
			SpawnThisResource = true;
		elseif (tblData.resource and tblData.SpawnWithArmy) then -- if this is an army-specific resource
			if (tblData.SpawnWithArmy == "ARMY_0") then
				SpawnThisResource = true;
			else
				for x, army in ListArmies() do -- loop through army list

					if (tblData.SpawnWithArmy == army) then -- if this army is present
						SpawnThisResource = true; -- spawn this resource
						break;
					end
				end
			end
		end

		if (SpawnThisResource) then -- if we can spawn the resource do it
			local bp, albedo, sx, sz, lod;
			if (tblData.type == "Mass") then
				albedo = "/env/common/splats/mass_marker.dds";
				bp = "/env/common/props/massDeposit01_prop.bp";
				sx = 2;
				sz = 2;
				lod = 100;
			else
				albedo = "/env/common/splats/hydrocarbon_marker.dds";
				bp = "/env/common/props/hydrocarbonDeposit01_prop.bp";
				sx = 6;
				sz = 6;
				lod = 200;
			end
			-- create the resource
			CreateResourceDeposit(tblData.type,	tblData.position[1], tblData.position[2], tblData.position[3], tblData.size);

			-- create the resource graphic on the map
			CreatePropHPR(bp, tblData.position[1], tblData.position[2], tblData.position[3], Random(0,360), 0, 0);

			-- create the resource icon on the map
			CreateSplat(
				tblData.position,           # Position
				0,                          # Heading (rotation)
				albedo,                     # Texture name for albedo
				sx, sz,                     # SizeX/Z
				lod,                        # LOD
				0,                          # Duration (0 == does not expire)
				-1,                         # army (-1 == not owned by any single army)
				0							# ???
			);
		end
	end
end






-- initializes the game settings
--------------------------------------------------------------------------
Survival_InitGame = function()

	LOG("----- Survival MOD: Configuring match settings...")
	-- check game configuration
	
	#####Checking difficulty options

			
	
	ScenarioInfo.Options.Victory = 'sandbox' -- force sandbox in order to implement our own rules
	local Armies = ListArmies()
	local ArmiesPresent = {}
	Survival_PlayerCount = table.getn(Armies) - 2 -- save player count, subtracting the 2 AI "players"

	-- loop through armies
	for i, Army in ListArmies() do
	
		if (Army == "ARMY_1") then
			ArmiesPresent[1] = true
		elseif (Army == "ARMY_2") then
			ArmiesPresent[2] = true
		elseif (Army == "ARMY_3") then
			ArmiesPresent[3] = true
		elseif	(Army == "ARMY_4") then
			ArmiesPresent[4] = true
		end
		-- check if it's a human army
		if (Army == "ARMY_1" or Army == "ARMY_2" or Army == "ARMY_3" or Army == "ARMY_4" or Army == "ARMY_5" or Army == "ARMY_6" or Army == "ARMY_7" or Army == "ARMY_8") then 

			ScenarioFramework.AddRestriction(Army, categories.WALL) -- don't allow them to build walls
			#ScenarioFramework.AddRestriction(Army, categories.AIR) -- don't allow them to build air stuff

			-- loop through other armies to ally with other human armies
			for x, ArmyX in ListArmies() do
				-- if human army
				if (ArmyX == "ARMY_1" or ArmyX == "ARMY_2" or ArmyX == "ARMY_3" or ArmyX == "ARMY_4" or ArmyX == "ARMY_5" or ArmyX == "ARMY_6" or ArmyX == "ARMY_7" or ArmyX == "ARMY_8") then 
					SetAlliance(Army, ArmyX, 'Ally') 
				end
			end			

			SetAlliance(Army, "ARMY_SURVIVAL_ALLY", 'Ally') -- friendly AI team
			SetAlliance(Army, "ARMY_SURVIVAL_ENEMY", 'Enemy')  -- enemy AI team

			SetAlliedVictory(Army, true) -- can win together of course :)
		end
	end

	
	SetAlliance("ARMY_SURVIVAL_ALLY", "ARMY_SURVIVAL_ENEMY", 'Enemy') -- the friendly and enemy AI teams should be enemies
	#SetIgnoreArmyUnitCap('ARMY_SURVIVAL_ENEMY', true) -- remove unit cap from enemy AI team
	SetArmyUnitCap("ARMY_SURVIVAL_ENEMY", 10000)
	SetArmyColor('ARMY_SURVIVAL_ALLY',150,150,0) 
	
	
		Survival_SpawnDef()
		LOG("Spawned def")
		Survival_SpawnTargets()
		LOG("Spawned Targets")
		SpawnDefenses(1)
		LOG("Spawned Defenses")
		ForkThread(Waypoint_Tracker)
		LOG("Issueing def move orders")
		ForkThread(AttackZoneCheck)
		LOG ("Checking attackzone")
		#Immunize_ACU() #Appears to freeze the game randomly

		ForkThread(SpawnEnemies, ArmiesPresent)
		LOG("Spawning Enemies")
		ForkThread(EmergencyShield) 
		LOG("Running Emergency Shield")
		ForkThread(Immunize_StartUnits)
		LOG("Immunizing units from friendly fire")
		
		##Will
		for n=1, 4 do
			if not ArmiesPresent[n] == true then
					ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ALLY','GROUP_' .. repr(n))
			end
		end
end


SpawnDefenses = function(CurrentArea)
	local Difficulty = ScenarioInfo.Options.Difficulty
	
	if Difficulty == nil then
		Difficulty = 4
	end
	LOG("Difficulty is " .. Difficulty)

	if Difficulty > 1 then
		ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ENEMY','GROUP_' .. (7+CurrentArea))
	end
	if Difficulty > 5 then
		ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ENEMY','GROUP_' .. (11+CurrentArea))
	end

	if Difficulty > 3 then
		ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ENEMY','GROUP_' .. (3+CurrentArea))
	end
	if Difficulty > 6 then
		ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ENEMY','GROUP_' .. (15+CurrentArea))
	end

	
	
end




------------------
Reinforcements = function ()

	local AllyID1 = 'xsa0102';
	local AllyID2 = 'xsa0103';
	local AllyID3 = 'xss0203';
	local ReinforcementStrength = ScenarioInfo.Options.Reinforcements;
	if ReinforcementStrength == nil then
		ReinforcementStrength = 1;
	end
	LOG("reinforcementstrenght is" .. ReinforcementStrength);
	local Quantity1 = 3 * ReinforcementStrength;
	local ArmyID = "ARMY_SURVIVAL_ALLY";
	local POS = ScenarioUtils.MarkerToPosition("WAYPOINT_0");
	
	local OrderID = 2;
	local PlatoonList = {};
		
	while not Survival_DefUnit:IsDead() do
		#LOG("starting spawning of allies");

		local PlatoonList = {};
		local Counter = 0;
		while Counter < Quantity1 do
			local NewUnit1 = CreateUnitHPR(AllyID1, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0);
			table.insert(PlatoonList, NewUnit1); -- add unit to a platoon
			Counter = Counter + 1;
			ImmunizeUnit(NewUnit1)
		end
		Counter = 0;
		while Counter < Quantity1/2 do
			local NewUnit1 = CreateUnitHPR(AllyID2, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0);
			table.insert(PlatoonList, NewUnit1); -- add unit to a platoon
			Counter = Counter + 1;
			ImmunizeUnit(NewUnit1)
		end
		local Counter = 0;
		while Counter < Quantity1/3 do
			local NewUnit1 = CreateUnitHPR(AllyID3, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0);
			table.insert(PlatoonList, NewUnit1); -- add unit to a platoon
			Counter = Counter + 1;
			ImmunizeUnit(NewUnit1)
		end
		local aiBrain = GetArmyBrain("ARMY_SURVIVAL_ALLY");
		local aiPlatoon = aiBrain:MakePlatoon('','');
		aiBrain:AssignUnitsToPlatoon(aiPlatoon, PlatoonList, 'Attack', 'None');
		#LOG("allies spawned");
		for n=1, TotalWaypoints do
			local TargetWaypoint = ScenarioUtils.MarkerToPosition("WAYPOINT_" .. n);
			aiPlatoon:AggressiveMoveToLocation(TargetWaypoint);
		end
		LOG("ally orders issued");
		#aiPlatoon:Patrol(Survival_DefUnit:GetPosition());	
		#if GetNextAttackZone() <= TotalTargets then
			aiPlatoon:GuardTarget(Survival_DefUnit);	
		#end
		WaitSeconds(30);
		local IdleUnits = GetArmyBrain("ARMY_SURVIVAL_ALLY"):GetListOfUnits(categories.MOBILE - categories.xss0302, true, false) 
		for _,unit in IdleUnits do
			if unit:IsIdleState() then
      	     ChangeUnitArmy(unit,"ARMY_" .. Random(1,Survival_PlayerCount))
			 end
		end
	end


end





##Manages what happens when a target dies
-------------
DeathOfTarget = function(self, instigator, type, overkillRatio)
	local CurrentAttackZone = (GetNextAttackZone() - 1)
	
	local Difficulty = ScenarioInfo.Options.Difficulty
	if Difficulty == nil then
		Difficulty = 4
	end

	Survival_DefUnit:GetWeaponByLabel('FrontTurret'):ChangeRateOfFire(.25)
	Survival_DefUnit:GetWeaponByLabel('MidTurret'):ChangeRateOfFire(.25)
	Survival_DefUnit:GetWeaponByLabel('BackTurret'):ChangeRateOfFire(.25)
	Survival_DefUnit:SetSpeedMult(.4)

	if CurrentAttackZone == 1 then
		ForkThread(SpeedBoost)
	end
	
			if CurrentAttackZone < (TotalTargets) then
				ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ENEMY','GROUP_' .. CurrentAttackZone)
				ScenarioFramework.SetPlayableArea('AREA_' .. (CurrentAttackZone + 1), false)
				ForkThread(Truck_PlatoonOrder)
				Survival_DefUnit:SetCustomName("The Weapon")
				BroadcastMSG("Target Command Center destroyed.  Moving to the next one.", 5)
				if Difficulty > 3 then
					SpawnBoss(Difficulty, CurrentAttackZone)
				end
				SpawnDefenses(CurrentAttackZone + 1)
			end
			if CurrentAttackZone == 3 then
				ForkThread(PickupUnits)
			end
			if CurrentAttackZone == (TotalTargets) and Difficulty <= 6 then
				BroadcastMSG("The human Coalition has been crushed.  There is nothing to stand in our way now.", 4)
				Survival_GameState = 2
				for i, army in ListArmies() do
					if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4" or army == "ARMY_5" or army == "ARMY_6" or army == "ARMY_7" or army == "ARMY_8") then
					GetArmyBrain(army):OnVictory()
					end
				end
				KillThread(CurrentThread(SpawnEnemies))
				KillThread(CurrentThread(Reinforcements))
				GetArmyBrain("ARMY_SURVIVAL_ENEMY"):OnDefeat()
			elseif CurrentAttackZone == (TotalTargets) and Difficulty >6 then
				ForkThread(SecretEnding)
				Survival_GameState = 2
			end
			
end

--------------------
SpeedBoost = function()
	local CurrentSpeed = .4
	while not Survival_DefUnit:IsDead() do
		local MaxHealth = Survival_DefUnit:GetMaxHealth()
		local Health1 = Survival_DefUnit:GetHealth()
		if Health1 < MaxHealth and CurrentSpeed > .4 then
			Survival_DefUnit:SetSpeedMult(.4)
			CurrentSpeed = .4
			BroadcastMSG("The Weapon is reporting damage.  Reverting power back to shields.", 4)
		elseif Health1 == MaxHealth and CurrentSpeed == .4 then
			WaitSeconds(10)
			Health1 = Survival_DefUnit:GetHealth()
			if Health1 < MaxHealth and CurrentSpeed > .4 then
				Survival_DefUnit:SetSpeedMult(.4)
				CurrentSpeed = .4
				BroadcastMSG("The Weapon is reporting damage.  Reverting power back to shields.", 4)
			elseif Health1 == MaxHealth then
			WaitSeconds(10)
			Health1 = Survival_DefUnit:GetHealth()
				if Health1 < MaxHealth and CurrentSpeed > .4  then
					Survival_DefUnit:SetSpeedMult(.4)
					BroadcastMSG("The Weapon is reporting damage.  Reverting power back to shields.", 4)
				elseif Health1 == MaxHealth then
					Survival_DefUnit:SetSpeedMult(.8)
					CurrentSpeed = .8
					BroadcastMSG("The Weapon is reporting 100% hull integrity.  Diverting power from shields to thrust.", 4)
				end
			end
		end
		WaitSeconds(10)
	end
end


##Monitors and conducts attack zone functions
--------------------------------------
AttackZoneCheck = function()
	local NextAttackZone = GetNextAttackZone()
	local TotalAttackZones = TotalTargets
	local OldDistance = 10000
	
	local FrontTurret = Survival_DefUnit:GetWeaponByLabel('FrontTurret')
	local MidTurret = Survival_DefUnit:GetWeaponByLabel('MidTurret')
	local BackTurret = Survival_DefUnit:GetWeaponByLabel('BackTurret')

	while not Survival_DefUnit:IsDead() and NextAttackZone <= TotalAttackZones  do
	
		local Distance = VDist3(Survival_DefUnit:GetPosition(),ScenarioUtils.MarkerToPosition("ATTACKZONE_" .. NextAttackZone))	
		#LOG("Next attack zone is attack zone " .. NextAttackZone .. " at distance " .. Distance )
		if Distance == OldDistance then
			LOG("Truck is stuck, re-issuing orders")
			ForkThread(Truck_PlatoonOrder)
		end
	
		while (Distance < 15) and not Survival_Target[NextAttackZone]:IsDead() do
			BroadcastMSG("Attack zone reached.  Targeting enemy Command Center.", 5)

			FrontTurret:ChangeRateOfFire(3.0)
			MidTurret:ChangeRateOfFire(2.5)
			BackTurret:ChangeRateOfFire(2.0)
			
			local aiBrain = GetArmyBrain('ARMY_SURVIVAL_ALLY') --"ARMY_SURVIVAL_ENEMY")
			local aiPlatoon = aiBrain:MakePlatoon('','')
			local PlatoonList = {};
			table.insert(PlatoonList, Survival_DefUnit);
			aiBrain:AssignUnitsToPlatoon(aiPlatoon, PlatoonList, 'Attack', 'None') -- platoon, unit list, "mission" and formation

			aiPlatoon:Stop()
			aiPlatoon:AttackTarget(Survival_Target[NextAttackZone])
			Survival_DefUnit:SetCustomName("SUPER CHARGED!!!")
			Survival_DefUnit:SetSpeedMult(.01)

			#FrontTurret:SetTargetEntity(Survival_Target[NextAttackZone])
			#MidTurret:SetTargetEntity(Survival_Target[NextAttackZone])
			#BackTurret:SetTargetEntity(Survival_Target[NextAttackZone])

			WaitSeconds(10)
		end
		OldDistance = Distance
		NextAttackZone = GetNextAttackZone()
		WaitSeconds(10)
		LOG("Next attack zone is attack zone " .. NextAttackZone .. " at distance " .. Distance )
	end

end



---------------------------------------
EmergencyShield = function()

	local	EmergencyShieldCount = nil

	local Difficulty = ScenarioInfo.Options.Difficulty
	
	if Difficulty == nil then
	Difficulty = 4
	end

	if Difficulty == 1 then
		EmergencyShieldCount = 5
	elseif Difficulty == 2 then 
		EmergencyShieldCount = 5
	elseif Difficulty == 3 then 
		EmergencyShieldCount = 4
	elseif Difficulty == 4 then 
		EmergencyShieldCount = 3
	elseif Difficulty == 5 then 
		EmergencyShieldCount = 3
	elseif Difficulty == 6 then 
		EmergencyShieldCount = 2
	elseif Difficulty == 7 then 
		EmergencyShieldCount = 2
	elseif Difficulty == 8 then 
		EmergencyShieldCount = 1
	else 
		EmergencyShieldCount = 1
	end

	LOG("Emergency Shield thread running with " .. EmergencyShieldCount .. " starting shields.")

		local bp ={
            ImpactEffects = 'SeraphimShieldHit01',
            ImpactMesh = '/effects/entities/ShieldSection01/ShieldSection01_mesh',
			Mesh = '/effects/entities/SeraphimShield01/SeraphimShield01_mesh',
            MeshZ = '/effects/entities/Shield01/Shield01z_mesh',
            RegenAssistMult = 60,
            ShieldEnergyDrainRechargeTime = 1,
            ShieldMaxHealth = 500000,
            ShieldRechargeTime = 1,
            ShieldRegenRate = 64,
            ShieldRegenStartTime = 1,
            ShieldSize = 20,
            ShieldVerticalOffset = -4,
	    }


	while not Survival_DefUnit:IsDead() do

		if Survival_DefUnit:GetHealth() < (Survival_DefUnit:GetMaxHealth() * .2) then
			BroadcastMSG("Our Weapon is CRITICALLY DAMAGED!", 4)
			if EmergencyShieldCount > 0 then
				EmergencyShieldCount = EmergencyShieldCount - 1
				BroadcastMSG("Emergency Shield activated.", 4)	
				ForkThread(function() WaitTicks(1) Survival_DefUnit:CreateShield(bp)   end)
				Survival_DefUnit:SetSpeedMult(0.01)
				
				local RegenAmount = (((Survival_DefUnit:GetMaxHealth() * .2) - Survival_DefUnit:GetHealth())/30)
				local Buff = import('/lua/sim/Buff.lua')
				if not Buffs['EmergencyRegen'] then
				BuffBlueprint {
                    Name = 'EmergencyRegen',
                    DisplayName = 'EmergencyRegen',
                    BuffType = 'ACUUPGRADEDMG',
                    Stacks = 'ALWAYS',
                    Duration = 30,
                    Affects = {
                        MaxHealth = {
                            Add = 000,
                            Mult = 1.0,
                        },
                        Regen = {
                            Add = RegenAmount,
                            Mult = 1.0,
                        },
                         },
                } 	
				end
				Buff.ApplyBuff(Survival_DefUnit, 'EmergencyRegen')   
				LOG("Regen amount for emer shield is " .. RegenAmount)
				WaitSeconds(30)
				BroadcastMSG("Emergency Shield dissipated.  We have " .. EmergencyShieldCount .. " remaining.",4)
	   	        Survival_DefUnit:DestroyShield()
				Survival_DefUnit:SetSpeedMult(0.4)
			end
		if EmergencyShieldCount == 0 then
		WaitSeconds(10)
		end
		end
		WaitSeconds(1)
	end

end


---------------------------------------------------------------------
function SpawnBoss(Difficulty, CurrentAttackZone)
	LOG("Start spawn boss with " .. Difficulty .. " diff at area " .. CurrentAttackZone)

	local Quantity = 1
	if Difficulty == 8 then
		Quantity = 2
	elseif Quantity == 9 then
		Quantity = 4
	end
	
	LOG("quantity is " .. Quantity)
	
	local UnitID = nil
	
	if CurrentAttackZone == 1 then
		UnitID = 'URS0302'
	elseif CurrentAttackZone == 2 then
		UnitID = 'UAS0401'
	elseif CurrentAttackZone == 3 then
		UnitID = 'XRL0403'
	elseif CurrentAttackZone == 4 then
		UnitID = nil
	end
			
	LOG("Boss unit is " .. UnitID)		
			
	local aiBrain = GetArmyBrain('ARMY_SURVIVAL_ENEMY') --"ARMY_SURVIVAL_ENEMY")
	local aiPlatoon = aiBrain:MakePlatoon('','')
	local POS = ScenarioUtils.MarkerToPosition("BOSSSPAWN_" .. CurrentAttackZone)
	local PlatoonList = {}
	
	#if not UnitID == nil then
		for n=1, (Quantity) do	
			local NewBoss = CreateUnitHPR(UnitID, 'ARMY_SURVIVAL_ENEMY', POS[1], (POS[2] + (Random(0,2) * 10)), POS[3], 0, Random(0,360),0)
			table.insert(PlatoonList, NewBoss); -- add unit to a platoon
			aiBrain:AssignUnitsToPlatoon(aiPlatoon, PlatoonList, 'Attack', 'None')
			aiPlatoon:AggressiveMoveToLocation(Survival_DefUnit:GetPosition())
			LOG("Boss unit has spawned")
		end	
	#end
end



---------------------------------------------------------------------
function SpawnEnemies(ArmiesPresent)

	#LOG("start spawn enemies")
	local WaypointsByAttackZone = {}
	for n=1,TotalTargets do
	WaypointsByAttackZone[n] = GetClosestWaypoint(ScenarioUtils.MarkerToPosition("ATTACKZONE_" .. repr(n)))
	LOG("Closest waypoint to attack zone " .. repr(n) .. " is waypoint " .. WaypointsByAttackZone[n])
	end
	
	local ReinforcementStrength = ScenarioInfo.Options.Reinforcements
	if ReinforcementStrength == nil then
		ReinforcementStrength = 1
	end
	if ReinforcementStrength > 0 then
		ScenarioUtils.CreateArmyGroup('ARMY_SURVIVAL_ALLY','GROUP_16')
		LOG("created group 16")
		ForkThread(Reinforcements)
	end
	
	WaitSeconds(15)

	local SpawnMultiplier = nil
	local WaveDelay = nil

	local Difficulty = ScenarioInfo.Options.Difficulty
	
	if Difficulty == nil then
	Difficulty = 4
	end


	
	if Difficulty == 1 then
		WaveDelay = 60
		SpawnMultiplier = .25
	elseif Difficulty == 2 then 
		WaveDelay = 55
		SpawnMultiplier = .5
	elseif Difficulty == 3 then 
		WaveDelay = 55
		SpawnMultiplier = .5
	elseif Difficulty == 4 then 
		WaveDelay = 50
		SpawnMultiplier = 1
	elseif Difficulty == 5 then 
		WaveDelay = 45
		SpawnMultiplier = 1
	elseif Difficulty == 6 then 
		WaveDelay = 45
		SpawnMultiplier = 1
	elseif Difficulty == 7 then 
		WaveDelay = 40
		SpawnMultiplier = 2
	elseif Difficulty == 8 then 
		WaveDelay = 35
		SpawnMultiplier = 2
	else 
		WaveDelay = 30
		SpawnMultiplier = 3
	end

	BroadcastMSG("The humans are retaliating.  Crush them.", 5)
	LOG("wave delay and spawn mult are " .. WaveDelay .. "/" .. SpawnMultiplier)
	
	local StartingAirFacs= {}
	local StartingLandFacs = {}
	local StartingNavalFacs = {}
	local StartingGates		= {}

	for n=0, TotalWaypoints do
	   local AirFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.LAND - categories.NAVAL, ScenarioUtils.MarkerToPosition("AIRSPAWN_" .. repr(n)), 30 )
   	   StartingAirFacs[n] = 0
   	   for _,unit in AirFacs do
       	         StartingAirFacs[n] = StartingAirFacs[n] + 1
  	   end
	   local LandFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.AIR - categories.NAVAL, ScenarioUtils.MarkerToPosition("GROUNDSPAWN_" .. repr(n)), 30 )
   	   StartingLandFacs[n] = 0
   	   for _,unit in LandFacs do
       	         StartingLandFacs[n] = StartingLandFacs[n] + 1
  	   end
	   local NavalFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.AIR - categories.LAND, ScenarioUtils.MarkerToPosition("WATERSPAWN_" .. repr(n)), 30 )
   	   StartingNavalFacs[n] = 0
   	   for _,unit in NavalFacs do
       	         StartingNavalFacs[n] = StartingNavalFacs[n] + 1
  	   end
	   local Gates =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.GATE, ScenarioUtils.MarkerToPosition("AMPHIBSPAWN_" .. repr(n)), 30 )
   	   StartingGates[n] = 0
   	   for _,unit in Gates do
       	         StartingGates[n] = StartingGates[n] + 1
  	   end
   	   LOG("Total air/land/naval/gates facs around spawn "..repr(n).." is ".. repr(StartingAirFacs[n]) .. "/" .. repr(StartingLandFacs[n]) .. "/" .. repr(StartingNavalFacs[n]) .. "/" .. repr(StartingGates[n]))
	end
	
		local ScaleDifficulty = ScenarioInfo.Options.ScaleDifficulty
		
		#1 for true, 2 for false
		if not ScaleDifficulty then
			ScaleDifficulty = 0
		end
		LOG("scaledifficulty is " .. ScaleDifficulty)
		if (ArmiesPresent[4] and ArmiesPresent[4] == true) or (ScaleDifficulty == 0) then
		ForkThread(SpawnGroundWave,0, SpawnMultiplier, WaypointsByAttackZone,WaveDelay)
		ForkThread(SpawnGroundWave,1, SpawnMultiplier, WaypointsByAttackZone,WaveDelay)
		end
		if (ArmiesPresent[3] and ArmiesPresent[3] == true) or (ScaleDifficulty == 0) then
		ForkThread(SpawnAirWave,0, SpawnMultiplier,WaypointsByAttackZone,WaveDelay)
		ForkThread(SpawnAirWave,1, SpawnMultiplier, WaypointsByAttackZone,WaveDelay)
		end
		if (ArmiesPresent[2] and ArmiesPresent[2] == true) or (ScaleDifficulty == 0) then
		ForkThread(SpawnAmphibWave,0, SpawnMultiplier,WaypointsByAttackZone,WaveDelay)	
		ForkThread(SpawnAmphibWave,1, SpawnMultiplier, WaypointsByAttackZone,WaveDelay)	
		end
		if (ArmiesPresent[1] and ArmiesPresent[1] == true) or (ScaleDifficulty == 0) then
		ForkThread(SpawnWaterWave,0, SpawnMultiplier, WaypointsByAttackZone,WaveDelay)
		ForkThread(SpawnWaterWave,1, SpawnMultiplier,WaypointsByAttackZone,WaveDelay)	
		end		
		#SpawnAirDrop(ClosestWaypoint, SpawnMultiplier, StartingGates[ ClosestWaypoint ])
		#SpawnArtilleryWave(ClosestWaypoint, SpawnMultiplier)		



		#WaitSeconds(WaveDelay)


end


---------------------------------------------------------------------------------------------
SpawnGroundWave = function(ClosestWaypointOffset, SpawnMultiplier,WaypointsByAttackZone,WaveDelay) -- This generates the units that will be in the wave then spawns them

	local UnitID1 = '0'
	local UnitID2 = '0'
	local UnitID3 = '0'
	local UnitID4 = '0'
	local Quantity1 = 0
	#local Quantity2 = 0
	local ClosestWaypoint = 0
	local StartingLandFacs = {}
	
	for n=1,TotalWaypoints do
		local LandFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.AIR - categories.NAVAL, ScenarioUtils.MarkerToPosition("GROUNDSPAWN_" .. repr(n)), 30 )
   	   StartingLandFacs[n] = 0
   	   for _,unit in LandFacs do
       	         StartingLandFacs[n] = StartingLandFacs[n] + 1
  	   end
	end
	
	while not Survival_DefUnit:IsDead() and ((GetClosestWaypoint() - ClosestWaypointOffset) >= 0) do
		ClosestWaypoint = GetClosestWaypoint() - ClosestWaypointOffset
	
		if ClosestWaypoint <= WaypointsByAttackZone[1] then
		UnitID1 = 'url0107'
		UnitID2 = 'url0107'
		UnitID3 = 'url0103'
		UnitID4 = 'url0202'
		Quantity1 = (((ClosestWaypoint + 1)) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[2] then
		UnitID1 = 'ual0202'
		UnitID2 = 'ual0103'
		UnitID3 = 'ual0307'
		UnitID4 = 'ual0111'
		Quantity1 = ((ClosestWaypoint - 1) * SpawnMultiplier)
		#Quantity2 = Quantity1/2	
		elseif ClosestWaypoint <= WaypointsByAttackZone[3] then
		UnitID1 = 'del0204'
		UnitID2 = 'uel0303'
		UnitID3 = 'uel0304'
		UnitID4 = 'uel0307'
		Quantity1 = (((ClosestWaypoint - 1)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[4] then
		UnitID1 = 'url0303'
		UnitID2 = 'uel0304'
		UnitID3 = 'xel0305'
		UnitID4 = 'ual0304'
		Quantity1 = (((ClosestWaypoint - 4)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		end

		local ArmyID = "ARMY_SURVIVAL_ENEMY"
		local POS = ScenarioUtils.MarkerToPosition("GROUNDSPAWN_" .. ClosestWaypoint)
		local OrderID = 2
		
		local FacsRemaining = GetFacsRemaining(ClosestWaypoint, categories.AIR, categories.NAVAL, "GROUND")
		if FacsRemaining < StartingLandFacs[ClosestWaypoint] and StartingLandFacs[ClosestWaypoint] > 0 then
			Quantity1 = Quantity1 * (FacsRemaining/StartingLandFacs[ClosestWaypoint])
			LOG("Using fac remaining groundspawn modifier value " .. FacsRemaining .. "/" .. StartingLandFacs[ClosestWaypoint])
		end
	
		local TargetLocation1 = ScenarioUtils.MarkerToPosition('ARMY_3')
		local TargetLocation2 = ScenarioUtils.MarkerToPosition('ARMY_4')	
	
		#LOG("Quantity of spawn is " .. Quantity)
	
		local Type = 'GROUND'
		Survival_SpawnGroundUnit(UnitID1, ArmyID, POS, OrderID, Quantity1, (Quantity1/2), UnitID2, UnitID3, UnitID4, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)
		WaitSeconds(WaveDelay  * .85)
	end
end

---------------------------------------------------------------------------------------------
SpawnAirWave = function(ClosestWaypointOffset, SpawnMultiplier,WaypointsByAttackZone,WaveDelay) -- This generates the units that will be in the wave then spawns them

	local UnitID1 = '0'
	local UnitID2 = '0'
	local UnitID3 = '0'
	local UnitID4 = '0'
	local Quantity1 = 0
	#local Quantity2 = 0
	local ClosestWaypoint = 0
	local StartingAirFacs = {}
	
	for n=1,TotalWaypoints do
		local AirFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.LAND - categories.NAVAL, ScenarioUtils.MarkerToPosition("AIRSPAWN_" .. repr(n)), 30 )
   	   StartingAirFacs[n] = 0
   	   for _,unit in AirFacs do
       	         StartingAirFacs[n] = StartingAirFacs[n] + 1
  	   end
	end
	
	while not Survival_DefUnit:IsDead() and ((GetClosestWaypoint() - ClosestWaypointOffset) >= 0) do
		ClosestWaypoint = GetClosestWaypoint() - ClosestWaypointOffset
	
		if ClosestWaypoint <= WaypointsByAttackZone[1] then
			UnitID1 = 'ura0102'
			UnitID2 = 'ura0103'
			UnitID3 = 'xra0105'
			UnitID4 = 'ura0102'
		Quantity1 = (((ClosestWaypoint + 1)) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[2] then
			UnitID1 = 'xaa0202'
			UnitID2 = 'xaa0202'
			UnitID3 = 'uaa0204'
			UnitID4 = 'uaa0103'
		Quantity1 = ((ClosestWaypoint -1 ) * SpawnMultiplier)
		#Quantity2 = Quantity1/2	
		elseif ClosestWaypoint <= WaypointsByAttackZone[3] then
			UnitID1 = 'uea0303'
			UnitID2 = 'uea0203'
			UnitID3 = 'dea0202'
			UnitID4 = 'uea0305'
		Quantity1 = (((ClosestWaypoint - 1)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[4] then
			UnitID1 = 'ura0303'
			UnitID2 = 'uea0304'
			UnitID3 = 'xaa0306'
			UnitID4 = 'xaa0305'
		Quantity1 = (((ClosestWaypoint - 4)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		end

	local ArmyID = "ARMY_SURVIVAL_ENEMY"
	local POS = ScenarioUtils.MarkerToPosition("AIRSPAWN_" .. ClosestWaypoint)
	local OrderID = 2
	
		local FacsRemaining = GetFacsRemaining(ClosestWaypoint, categories.LAND, categories.NAVAL, "AIR")
		if FacsRemaining < StartingAirFacs and StartingAirFacs[ClosestWaypoint] > 0 then
			Quantity1 = Quantity1 * (FacsRemaining/StartingAirFacs[ClosestWaypoint])
			LOG("Using fac remaining airspawn modifier value " .. FacsRemaining .. "/" .. StartingAirFacs[ClosestWaypoint])
		end
	
		local TargetLocation1 = Survival_DefUnit:GetPosition()
		local TargetLocation2 = ScenarioUtils.MarkerToPosition('ARMY_' .. Random(1,4))	
	
		#LOG("Quantity of spawn is " .. Quantity)
	
		local Type = 'AIR'
		Survival_SpawnGroundUnit(UnitID1, ArmyID, POS, OrderID, Quantity1, (Quantity1/2), UnitID2, UnitID3, UnitID4, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)
		WaitSeconds(WaveDelay * .75)
	end
end



---------------------------------------------------------------------------------------------
SpawnAmphibWave = function(ClosestWaypointOffset, SpawnMultiplier,WaypointsByAttackZone,WaveDelay) -- This generates the units that will be in the wave then spawns them

	local UnitID1 = '0'
	local UnitID2 = '0'
	local UnitID3 = '0'
	local UnitID4 = '0'
	local Quantity1 = 0
	#local Quantity2 = 0
	local ClosestWaypoint = 0
	local StartingGates = {}
	
	for n=1,TotalWaypoints do
		local Gates =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.GATE, ScenarioUtils.MarkerToPosition("AMPHIBSPAWN_" .. repr(n)), 30 )
   	   StartingGates[n] = 0
   	   for _,unit in Gates do
       	         StartingGates[n] = StartingGates[n] + 1
  	   end
	end
	
	while not Survival_DefUnit:IsDead() and ((GetClosestWaypoint() - ClosestWaypointOffset) >= 0) do
		ClosestWaypoint = GetClosestWaypoint() - ClosestWaypointOffset
	
		if ClosestWaypoint <= WaypointsByAttackZone[1] then
		UnitID1 = 'url0203'
		UnitID2 = 'url0203'
		UnitID3 = 'url0203'
		UnitID4 = 'url0203'
		Quantity1 = (((ClosestWaypoint + 1)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[2] then
		UnitID1 = 'xal0203'
		UnitID2 = 'xal0203'
		UnitID3 = 'ual0307'
		UnitID4 = 'dal0310'
		Quantity1 = ((ClosestWaypoint) * SpawnMultiplier)
		#Quantity2 = Quantity1/2	
		elseif ClosestWaypoint <= WaypointsByAttackZone[3] then
		UnitID1 = 'uel0203'
		UnitID2 = 'uel0203'
		UnitID3 = 'xel0305'
		UnitID4 = 'xel0305'
		Quantity1 = (((ClosestWaypoint - 1)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[4] then
		UnitID1 = 'xrl0305'
		UnitID2 = 'uel0203'
		UnitID3 = 'urs0201'
		UnitID4 = 'dal0310'
		Quantity1 = (((ClosestWaypoint - 4)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		end

		local ArmyID = "ARMY_SURVIVAL_ENEMY"
		local POS = ScenarioUtils.MarkerToPosition("AMPHIBSPAWN_" .. ClosestWaypoint)
		local OrderID = 2
	
		local GatesRemaining = table.getn(AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.GATE, ScenarioUtils.MarkerToPosition("AMPHIBSPAWN_" .. ClosestWaypoint), 30 ))
		if GatesRemaining < StartingGates[ClosestWaypoint] and StartingGates[ClosestWaypoint] > 0 then
			Quantity1 = Quantity1 * (GatesRemaining/StartingGates[ClosestWaypoint])
			LOG("Using fac remaining amphibspawn modifier value " .. GatesRemaining .. "/" .. StartingGates[ClosestWaypoint])
		end
	
		local TargetLocation1 = ScenarioUtils.MarkerToPosition('ARMY_1')
		local TargetLocation2 = ScenarioUtils.MarkerToPosition('ARMY_2')	
	
		#LOG("Quantity of spawn is " .. Quantity)
	
		local Type = 'AMPHIB'
		Survival_SpawnGroundUnit(UnitID1, ArmyID, POS, OrderID, Quantity1, (Quantity1/2), UnitID2, UnitID3, UnitID4, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)
		WaitSeconds(WaveDelay * 1.8)
	end
end



---------------------------------------------------------------------------------------------
SpawnWaterWave = function(ClosestWaypointOffset, SpawnMultiplier,WaypointsByAttackZone,WaveDelay) -- This generates the units that will be in the wave then spawns them

	local UnitID1 = '0'
	local UnitID2 = '0'
	local UnitID3 = '0'
	local UnitID4 = '0'
	local Quantity1 = 0
	#local Quantity2 = 0
	local ClosestWaypoint = 0
	local StartingNavalFacs = {}
	
	for n=1,TotalWaypoints do
		local NavalFacs =  AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - categories.AIR - categories.LAND, ScenarioUtils.MarkerToPosition("WATERSPAWN_" .. repr(n)), 30 )
   	   StartingNavalFacs[n] = 0
   	   for _,unit in NavalFacs do
       	         StartingNavalFacs[n] = StartingNavalFacs[n] + 1
  	   end
	end
	
	while not Survival_DefUnit:IsDead() and ((GetClosestWaypoint() - ClosestWaypointOffset) >= 0) do
		ClosestWaypoint = GetClosestWaypoint() - ClosestWaypointOffset
	
		if ClosestWaypoint <= WaypointsByAttackZone[1] then
	UnitID1 = 'urs0203'
	UnitID2 = 'urs0103'
	UnitID3 = 'urs0103'
	UnitID4 = 'xrs0205'
		Quantity1 = (((ClosestWaypoint + 1)) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[2] then
	UnitID1 = 'uas0103'
	UnitID2 = 'uas0203'
	UnitID3 = 'uas0202'
	UnitID4 = 'uas0201'
		Quantity1 = ((ClosestWaypoint -1 ) * SpawnMultiplier)
		#Quantity2 = Quantity1/2	
		elseif ClosestWaypoint <= WaypointsByAttackZone[3] then
	UnitID1 = 'ues0202'
	UnitID2 = 'xes0102'
	UnitID3 = 'ues0201'
	UnitID4 = 'xes0307'
		Quantity1 = (((ClosestWaypoint - 1)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		elseif ClosestWaypoint <= WaypointsByAttackZone[4] then
	UnitID1 = 'xrs0204'
	UnitID2 = 'uas0201'
	UnitID3 = 'xes0307'
	UnitID4 = 'urs0307'
		Quantity1 = (((ClosestWaypoint - 4)/2) * SpawnMultiplier)
		#Quantity2 = Quantity1/2
		end

		local ArmyID = "ARMY_SURVIVAL_ENEMY"
		local POS = ScenarioUtils.MarkerToPosition("WATERSPAWN_" .. ClosestWaypoint)
		local OrderID = 2
	
		local FacsRemaining = GetFacsRemaining(ClosestWaypoint, categories.AIR, categories.LAND, "WATER")
		if FacsRemaining < StartingNavalFacs[ClosestWaypoint] and StartingNavalFacs[ClosestWaypoint] > 0 then
			Quantity1 = Quantity1 * (FacsRemaining/StartingNavalFacs[ClosestWaypoint])
			LOG("Using fac remaining waterspawn modifier value " .. FacsRemaining .. "/" .. StartingNavalFacs[ClosestWaypoint])
		end
	
		local TargetLocation1 = Survival_DefUnit:GetPosition()
		local TargetLocation2 = ScenarioUtils.MarkerToPosition('ARMY_' .. Random(1,4))		
	
		#LOG("Quantity of spawn is " .. Quantity)
	
		local Type = 'WATER'
		Survival_SpawnGroundUnit(UnitID1, ArmyID, POS, OrderID, Quantity1, (Quantity1/2), UnitID2, UnitID3, UnitID4, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)
		WaitSeconds(WaveDelay * 1.5)
	end
end



#spawns all units
------------
Survival_SpawnGroundUnit = function(BP1, ArmyID, POS, OrderID, Quantity1, Quantity2, BP2, BP3, BP4, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)
	local PlatoonList = {}
	local Counter = 0
	local ArmyID = "ARMY_SURVIVAL_ENEMY"
	#LOG("quantity of spawn at spawn " .. ClosestWaypoint .. " is " .. Quantity1)

	while Counter < Quantity1 do
		table.insert(PlatoonList, CreateUnitHPR(BP1, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0)) 
		Survival_PlatoonOrder(ArmyID, PlatoonList, OrderID, TargetLocation1, TargetLocation2, ClosestWaypoint, Type) 
		Counter = Counter + 1
	end
	Counter = 0
	while Counter < Quantity1  do
		table.insert(PlatoonList, CreateUnitHPR(BP2, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0)) 
		Survival_PlatoonOrder(ArmyID, PlatoonList, OrderID, TargetLocation1, TargetLocation2, ClosestWaypoint, Type) 
		Counter = Counter + 1
	end
	Counter = 0
	while Counter < Quantity1 do
		table.insert(PlatoonList, CreateUnitHPR(BP3, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0)) 
		Survival_PlatoonOrder(ArmyID, PlatoonList, OrderID, TargetLocation1, TargetLocation2, ClosestWaypoint, Type) 
		Counter = Counter + 1
	end
	Counter = 0
	while Counter < Quantity1 do
		table.insert(PlatoonList, CreateUnitHPR(BP4, ArmyID, POS[1], POS[2], POS[3], 0, Random(0,360),0)) 
		Survival_PlatoonOrder(ArmyID, PlatoonList, OrderID, TargetLocation1, TargetLocation2, ClosestWaypoint, Type) 
		Counter = Counter + 1
	end

end
--------------------------------------------------------------------------
Survival_PlatoonOrder = function(ArmyID, UnitList, OrderID, TargetLocation1, TargetLocation2, ClosestWaypoint, Type)	

--	LOG("----- Survival MOD: PLATOON: Start function...")

	if (UnitList == nil) then
		return
	end

	local aiBrain = GetArmyBrain(ArmyID) --"ARMY_SURVIVAL_ENEMY")
	local aiPlatoon = aiBrain:MakePlatoon('','')
	aiBrain:AssignUnitsToPlatoon(aiPlatoon, UnitList, 'Attack', 'None') -- platoon, unit list, "mission" and formation
	
	##To prevent naval units from camping off-map
	if Type == 'WATER' then		
		aiPlatoon:MoveToLocation(ScenarioUtils.MarkerToPosition("WAYPOINT_" .. (ClosestWaypoint + 1)), false)
	end
	
	##This is to enable ground units to attack from more than one direction.
	if Random(1,2) == 1 and Type == 'GROUND' then
		aiPlatoon:AggressiveMoveToLocation(ScenarioUtils.MarkerToPosition(Type .. "WAYPOINT_" .. (ClosestWaypoint)))
	end
	if TargetLocation2 then
		if Random(1,2) == 1 then
		aiPlatoon:AggressiveMoveToLocation(TargetLocation1)
		else
		aiPlatoon:AggressiveMoveToLocation(TargetLocation2)
		end
	else
	aiPlatoon:MoveToLocation(TargetLocation1, false)
	end
	###This is to give units something to do if they reach the original area without finding a target
	if Type == 'AIR' then
		aiPlatoon:Patrol(ScenarioUtils.MarkerToPosition("WAYPOINT_1"))
		for n=1,TotalTargets do
		aiPlatoon:Patrol(ScenarioUtils.MarkerToPosition("ATTACKZONE_" .. repr(n)))
		end
	end
	if Type == 'AMPHIB' then
		aiPlatoon:Patrol(Survival_DefUnit:GetPosition())
	end
	
end


----------------------------
function GetNextAttackZone()
	local AttackZoneCounter = 1
	while AttackZoneCounter <= TotalTargets and Survival_Target[AttackZoneCounter]:IsDead() do
		AttackZoneCounter = AttackZoneCounter + 1
	end
	return AttackZoneCounter
end

----------------------------
function GetClosestWaypoint(POS)
	if POS == nil then
	POS = Survival_DefUnit:GetPosition()
	end

	local WaypointCounter = 0
	local ShortestDistance = 20000
	local ShortestDistanceWaypoint = 100
	local WaypointDistance = 1000

	while (WaypointCounter < (TotalWaypoints + 1)) do
		WaypointDistance = VDist3(POS,ScenarioUtils.MarkerToPosition("WAYPOINT_" .. WaypointCounter))
		if WaypointDistance < ShortestDistance then
			ShortestDistance = WaypointDistance
			ShortestDistanceWaypoint = WaypointCounter
		end
	WaypointCounter = WaypointCounter + 1
	end
	#LOG("returned GetClosestWaypoint")
	return ShortestDistanceWaypoint
end








--------------------------------------------------------------------------
Survival_SpawnDef = function()

	LOG("----- Survival MOD: Initializing defense object...")

	local POS = ScenarioUtils.MarkerToPosition("WAYPOINT_0")

	Survival_DefUnit = CreateUnitHPR('XSS0302', "ARMY_SURVIVAL_ALLY", POS[1], POS[2], POS[3], 0,90,0)  
	
	Survival_DefUnit:SetReclaimable(false)
	Survival_DefUnit:SetCapturable(false)
	Survival_DefUnit:SetSpeedMult(0.4)
	Survival_DefUnit:SetTurnMult(0.2) 
	
	Survival_DefUnit:GetWeaponByLabel('AntiAirLeft'):ChangeRateOfFire(2)
	Survival_DefUnit:GetWeaponByLabel('AntiAirLeft'):AddDamageMod(12) 
	Survival_DefUnit:GetWeaponByLabel('AntiAirRight'):ChangeRateOfFire(2)
	Survival_DefUnit:GetWeaponByLabel('AntiAirRight'):AddDamageMod(12) 
	Survival_DefUnit:SetCustomName("The Weapon")


	local Buff = import('/lua/sim/Buff.lua')
	
        if not Buffs['SeraphimACUDamageStabilization'] then
               BuffBlueprint {
                    Name = 'SeraphimACUDamageStabilization',
                    DisplayName = 'SeraphimACUDamageStabilization',
                    BuffType = 'ACUUPGRADEDMG',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = 0,
                            Mult = 1.0,
                        },
                        Regen = {
                            Add = 50,
                            Mult = 1.0,
                        },
                         },
                } 	
	end
    Buff.ApplyBuff(Survival_DefUnit, 'SeraphimACUDamageStabilization')    
	
	Survival_DefUnit:SetIntelRadius('Vision', 50)
	Survival_DefUnit:SetIntelRadius('Radar', 150)

	-- when the def object dies
	Survival_DefUnit.OldOnKilled = Survival_DefUnit.OnKilled
	 
	Survival_DefUnit.OnKilled = function(self, instigator, type, overkillRatio)

		BroadcastMSG("The Weapon has been lost.  We have no choice but to retreat.  Prepare for recall.", 8)
		self.OldOnKilled(self, instigator, type, overkillRatio)

		Survival_GameState = 3
		KillThread(CurrentThread(Reinforcements))
		GetArmyBrain("ARMY_SURVIVAL_ALLY"):OnDefeat()
		
		for i, army in ListArmies() do

			if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4" or army == "ARMY_5" or army == "ARMY_6" or army == "ARMY_7" or army == "ARMY_8") then
				--killAllCommanders(army)
				GetArmyBrain(army):OnDefeat()
			end
		end
	end

	#Survival_DefLastHP = Survival_DefUnit:GetHealth()
	LOG("Spawn def complete")


end




-- spawns target objects
--------------------------------------------------------------------------
Survival_SpawnTargets = function()

	LOG("----- Survival MOD: Initializing targets...")

	Survival_Target = {}

	local bp ={
            ImpactEffects = 'UEFShieldHit01',
            ImpactMesh = '/effects/entities/ShieldSection01/ShieldSection01_mesh',
            Mesh = '/effects/entities/Shield01/Shield01_mesh',
            MeshZ = '/effects/entities/Shield01/Shield01z_mesh',
            RegenAssistMult = 60,
            ShieldEnergyDrainRechargeTime = 5,
            ShieldMaxHealth = 50000,
            ShieldRechargeTime = 30,
            ShieldRegenRate = 1000,
            ShieldRegenStartTime = 1,
            ShieldSize = 50,
            ShieldVerticalOffset = -7,
	}



	for n=1, TotalTargets do
		local POS = ScenarioUtils.MarkerToPosition("TARGET_" .. n)

		if n==1 then
			Survival_Target[n] = CreateUnitHPR('URC1301', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], POS[3], 0,0,0)  
			bp.ImpactEffects = 'CybranShieldHit01'
			bp.Mesh = '/effects/entities/CybranShield01/CybranShield01_mesh'
			Survival_Target[n]:SetCustomName("Cybran Command Center")
		end	
		if n==2 then
			Survival_Target[n] = CreateUnitHPR('UAC1301', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], POS[3], 0,0,0)  
			bp.ImpactEffects = 'AeonShieldHit01'
			bp.Mesh = '/effects/entities/AeonShield01/AeonShield01_mesh'
			Survival_Target[n]:SetCustomName("Aeon Command Center")
		end
		if n==3 then
			Survival_Target[n] = CreateUnitHPR('XEC1401', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], POS[3], 0,0,0)
			bp.ImpactEffects = 'UEFShieldHit01'
			bp.Mesh = '/effects/entities/Shield01/Shield01_mesh'	
			Survival_Target[n]:SetCustomName("UEF Command Center")			
		end	
		if n==TotalTargets then
			Survival_Target[n] = CreateUnitHPR('XEC1301', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], POS[3], 0,0,0)  
			Survival_Target[n]:SetCustomName("Coalition HQ")
			bp.ShieldMaxHealth = 100000
			bp.ImpactEffects = 'UEFShieldHit01'
			bp.Mesh = '/effects/entities/Shield01/Shield01_mesh'	
			Survival_Target[n]:SetReclaimable(false)
			Survival_Target[n]:SetCapturable(false)
		end

		Survival_Target[n]:SetReclaimable(false)
		Survival_Target[n]:SetCapturable(false)
		Survival_Target[n]:CreateShield(bp)

		local Buff = import('/lua/sim/Buff.lua')
              	BuffBlueprint {
                 	  Name = 'SeraphimACUDamageStabilization',
                   	  DisplayName = 'SeraphimACUDamageStabilization',
                  	  BuffType = 'ACUUPGRADEDMG',
                  	  Stacks = 'ALWAYS',
                  	  Duration = -1,
                  	  Affects = {
                   	     MaxHealth = {
                   	         Add = 7500,
                       	     Mult = 1.0,
                       	 },
                       	 Regen = {
                       	     Add = 50,
                       	     Mult = 1.0,
                       	 },
                       	 },
                } 	
		Buff.ApplyBuff(Survival_Target[n], 'SeraphimACUDamageStabilization')    
		Survival_Target[n].OldOnKilled = Survival_Target[n].OnKilled
	 	Survival_Target[n].OnKilled = function(self, instigator, type, overkillRatio)
			self.OldOnKilled(self, instigator, type, overkillRatio)
			DeathOfTarget(self, instigator, type, overkillRatio)
		end
	end
			LOG("Spawn targets complete")
end

function PickupUnits(pickup, dropoff)
   local aiBrain = GetArmyBrain("ARMY_SURVIVAL_ENEMY")
   local NextAttackZone = GetNextAttackZone()
   while NextAttackZone == TotalTargets do
      for n=1, 3 do
     local DropOff = ScenarioUtils.MarkerToPosition("DROPOFF_" .. repr(n))
      local PickUp = ScenarioUtils.MarkerToPosition("PICKUP_" .. repr(n))
      if not HumanUnitsClose(PickUp) then
         local unitlist = AIUtils.GetOwnUnitsAroundPoint(aiBrain, categories.LAND, PickUp, 30 )
         if table.getn(unitlist) > 0 then
               LOG("Dispatching Transports")
            local capacity = 0
            local shipment = {}
            local shipmentlist = {}
            local transportlist = {}
            local platoon = aiBrain:MakePlatoon('','')
            aiBrain:AssignUnitsToPlatoon(platoon, unitlist, 'Attack', 'None')
            -- distribute Units to transports
            for k, unit in unitlist do
               if EntityCategoryContains(categories.TECH1, unit) then
                  capacity = capacity + 1
               elseif EntityCategoryContains(categories.TECH2, unit) then
                  capacity = capacity + 2
               elseif EntityCategoryContains(categories.TECH3, unit) then
                  capacity = capacity + 4
               end
      #local DropOff = ScenarioUtils.MarkerToPosition("DROPOFF_" .. Random(1,3))
               if capacity > 24 then
                  -- new unit doesn't fit anymore, send transport for this shipment
              local TransSpawn = ScenarioUtils.MarkerToPosition("TRANSSPAWN_" .. Random(1,3))
                  local transport = CreateUnitHPR("xea0306", 'ARMY_SURVIVAL_ENEMY', TransSpawn[1], TransSpawn[2], TransSpawn[3], 0, 0, 0)
                  table.insert(transportlist, transport)                 
                  table.insert(shipmentlist, table.copy(shipment))
                  shipment = {}
           --       IssueTransportLoad(shipment, transport)
           --       IssueTransportUnload(transport, DropOff)
                  capacity = capacity - 24
               end
               table.insert(shipment, unit)
            end
            -- load the final batch of units
                  local TransSpawn = ScenarioUtils.MarkerToPosition("TRANSSPAWN_" .. Random(1,3))
            local transport = CreateUnitHPR("xea0306", 'ARMY_SURVIVAL_ENEMY',TransSpawn[1], TransSpawn[2], TransSpawn[3],0, 0, 0)
            table.insert(transportlist, transport)
            table.insert(shipmentlist, shipment)
           
            local transportplatoon = aiBrain:MakePlatoon('','')
            aiBrain:AssignUnitsToPlatoon(transportplatoon, transportlist, 'Attack', 'None')
            transportplatoon:MoveToLocation(PickUp, false)

            WaitSeconds(10)
            transportplatoon:Stop()
                       
            local shipmentsalive = {}
            local transportsalive = {}
            for i = 1, table.getn(shipmentlist) do
               -- Check if transport's still alive
               if not transportlist[i]:IsDead() then
                  table.insert(transportsalive, transportlist[i])
               end
               -- Check if units are still alive
               local unitsalive = {}
               for k, unit in shipmentlist[i] do
                  if not unit:IsDead() then
                     table.insert(unitsalive, unit)
                  end
               end
               if table.getn(unitsalive) > 0 then
                  table.insert(shipmentsalive, table.copy(unitsalive))
               end
            end
           
            for i = 1, math.min(table.getn(shipmentsalive), table.getn(transportsalive)) do
                 IssueTransportLoad(shipmentsalive[i], transportsalive[i])
            end
            if table.getn(transportsalive) > 0 then
               IssueTransportUnload( transportsalive, DropOff)
               if table.getn(shipmentsalive) > 0 then
                  platoon:MoveToLocation(ScenarioUtils.MarkerToPosition("ARMY_" .. Random(1,4)), false)
               end
               transportplatoon:AttackTarget(Survival_DefUnit)
            end

            -- ToDo fly the Transports off the map and destroy them.
         end
      end
      WaitSeconds(30)
      end
   end
end

function HumanUnitsClose(pos)

   for x, army in ListArmies() do
      if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4") then
         local unitlist = AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain(army), categories.LAND + categories.STRUCTURE, pos, 30 )
         if table.getn(unitlist) > 0 then
            -- Human Units found
            return true
         end
      end
   end
   
   return false
end


----------------------------------------------------------------------------
BroadcastMSG = function(MSG, Fade, TextColor)
	PrintText(MSG, 20, TextColor, Fade, 'center')	
end


------------------------------------------
GetFacsRemaining = function(WaypointNumber, TypeNot1, TypeNot2, SpawnType)
      local units = AIUtils.GetOwnUnitsAroundPoint(GetArmyBrain("ARMY_SURVIVAL_ENEMY"), categories.FACTORY - TypeNot1 - TypeNot2, ScenarioUtils.MarkerToPosition(SpawnType .. "SPAWN_" .. WaypointNumber), 30 )
	  LOG("GetFacsRemaining returning value " .. table.getn(units) .. " for " .. SpawnType)
      return table.getn(units) 
end

-- keeps the bship moving
--------------------------------------------------------------------------

Waypoint_Tracker = function(UnitID, ArmyID, POS, OrderID)


	#Handle the initial time before the unit arrives
	local StartTime = ScenarioInfo.Options.StartTime
	if StartTime == nil then
	StartTime = 0
	end
	BroadcastMSG("The Weapon will arrive in " .. (StartTime + 5) .. " seconds.", 5)	

	Survival_DefUnit:SetSpeedMult(0.01)
	for i = 1, Survival_DefUnit:GetWeaponCount() do
	   Survival_DefUnit:GetWeapon(i):SetEnabled(false)
    end 
	WaitSeconds(StartTime+1)
	Survival_DefUnit:SetSpeedMult(0.4)
	for i = 1, Survival_DefUnit:GetWeaponCount() do
		#Survival_DefUnit:SetWeaponEnabled(i, true)
	   Survival_DefUnit:GetWeapon(i):SetEnabled(true)
    end 
	


	Truck_PlatoonOrder() -- give the unit orders
	#WaitSeconds(10)
end




##issues specific orders to the truck, will que orders until no more waypoints are left
----------------------------------------------------------------------------------
Truck_PlatoonOrder = function()
	LOG("Start truck platoon order")
	#if (UnitList == nil) then
	#	return
	#end

	#local UnitList = {}
	local WaypointNumber = GetClosestWaypoint()
	local TargetWaypoint = ScenarioUtils.MarkerToPosition("WAYPOINT_" .. (WaypointNumber + 1))
	
	local PlatoonList = {}
	table.insert(PlatoonList, Survival_DefUnit) -- add unit to a platoon
	local aiBrain = GetArmyBrain("ARMY_SURVIVAL_ALLY") #GetArmyBrain(ArmyID)	--"ARMY_SURVIVAL_ENEMY")
	local aiPlatoon = aiBrain:MakePlatoon('','')
	aiBrain:AssignUnitsToPlatoon(aiPlatoon, PlatoonList, 'Attack', 'None') -- platoon, unit list, "mission" and 
	aiPlatoon:Stop()
	
	BroadcastMSG("The Weapon is moving.  Defend it.", 5)
	
	while WaypointNumber < TotalWaypoints do
		WaypointNumber = WaypointNumber + 1
		TargetWaypoint = ScenarioUtils.MarkerToPosition("WAYPOINT_" .. WaypointNumber)
		#local POS = ScenarioUtils.MarkerToPosition("WAYPOINT_1") 
		aiPlatoon:MoveToLocation(TargetWaypoint, false)
		LOG("Bship has qued a move order to waypoint number " .. WaypointNumber)
		WaitSeconds(1)	
	end
	LOG("End truck platoon order")
end

#handles the secret ending (shhhhh)
-------------
SecretEnding = function()
 	ScenarioFramework.SetPlayableArea('AREA_' .. (TotalTargets + 1), false)
	BroadcastMSG("Well done. The Coalition HQ has been dest... wait.... WHAT IS THAT?", 4)
	
	local POS = ScenarioUtils.MarkerToPosition("SECRETSPAWN")
	CreateUnitHPR('UEC1101', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], (POS[3] - 2), 0,0,0)

	POS2 = ScenarioUtils.MarkerToPosition("SECRETWAYPOINT")
	SecretGate = CreateUnitHPR('UEB0304', "ARMY_SURVIVAL_ENEMY", POS2[1], POS2[2], POS2[3], 0,0,0)
	
	local Survival_DefUnit2 = {}

	for n=1,3 do
	Survival_DefUnit2[n] = CreateUnitHPR('UEC0001', "ARMY_SURVIVAL_ENEMY", POS[1], POS[2], POS[3], 0,0,0)
	ForkThread(EndTeleport, Survival_DefUnit2[n], POS2)
	#WaitSeconds(2)
	end

	WaitSeconds(15)
	BroadcastMSG("The Coalition is escaping! Stop Them!",5)
	BroadcastMSG("The Coalition is escaping! Stop Them!",5)
	WaitSeconds(15)
	BroadcastMSG("Find out where they went and assault them!  We can't let the coalition escape!", 4)

	
				for i, army in ListArmies() do
					if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4" or army == "ARMY_5" or army == "ARMY_6" or army == "ARMY_7" or army == "ARMY_8") then
					GetArmyBrain(army):OnVictory()
					end
				end
				KillThread(CurrentThread(SpawnEnemies))
				KillThread(CurrentThread(Reinforcements))
				GetArmyBrain("ARMY_SURVIVAL_ENEMY"):OnDefeat()
end

EndTeleport = function(unit, POS)
	local PlatoonList = {}
	local aiBrain = GetArmyBrain("ARMY_SURVIVAL_ENEMY") 
	local aiPlatoon = aiBrain:MakePlatoon('','')
	unit:SetReclaimable(false);
	unit:SetCapturable(false);
	#unit:SetSpeedMult(.7);
	unit:SetCustomName("Coalition VIP")
	table.insert(PlatoonList, unit) -- add unit to a platoon
	aiBrain:AssignUnitsToPlatoon(aiPlatoon, PlatoonList, 'Attack', 'None')
	aiPlatoon:MoveToLocation(POS2, false)	
	
	while not unit:IsDead() do
		if VDist3(unit:GetPosition(),POS) < 1 then
			LOG("Playing scounds and teleport stuff")
			unit:PlayUnitSound('CommanderArrival')
			unit:CreateProjectile( '/effects/entities/UnitTeleport01/UnitTeleport01_proj.bp', 0, 1.35, 0, nil, nil, nil):SetCollision(false)
			unit:Destroy()
		end
	WaitTicks(5)
	end

end

##prevents units from hitting friendly units
------------------------------------------------------------------------
Immunize_StartUnits = function()
   WaitSeconds(5)
   for i, army in ListArmies() do
      if (army == "ARMY_1" or army == "ARMY_2" or army == "ARMY_3" or army == "ARMY_4" or army == "ARMY_SURVIVAL_ALLY") then
         local unitList = GetArmyBrain(army):GetListOfUnits( categories.ALLUNITS , false, false)
         for j, unit in unitList do
            ImmunizeUnit(unit)
         end
      end   
   end
end

## Makes a unit immune to friendly fire and neutralizes its weapons to not harm allies
ImmunizeUnit = function(unit)
   if not unit.immune == 1 then
      unit.immune = 1
      -- Makes friendly unit's projectiles pass through this unit
      unit.OldOnCollisionCheck = unit.OnCollisionCheck
      unit.OnCollisionCheck = newOnCollisionCheck
      unit.OldOnCollisionCheckWeapon = unit.OnCollisionCheckWeapon
      unit.OnCollisionCheckWeapon = newOnCollisionCheckWeapon
      
      -- Marks this unit's weapons as friendly
      neutralizeWeapons(unit)
      
      -- Modify unit's build function to pass immunity on to other units
      if unit.OnStartBuild and unit.OldOnStartBuild == nil then
         unit.OldOnStartBuild = unit.OnStartBuild
         unit.OnStartBuild = newOnStartBuild
      end
   end
end

## Immunize units built by this unit
newOnStartBuild = function(self, unitBeingBuilt, order)
   self:OldOnStartBuild(unitBeingBuilt, order )
   
   if unitBeingBuilt.immune == nil then
      -- Makes friendly unit's projectiles pass through UnitBeingBuilt
      unitBeingBuilt.immune = 1
      unitBeingBuilt.OldOnCollisionCheck = unitBeingBuilt.OnCollisionCheck
      unitBeingBuilt.OnCollisionCheck = newOnCollisionCheck
      unitBeingBuilt.OldOnCollisionCheckWeapon = unitBeingBuilt.OnCollisionCheckWeapon
      unitBeingBuilt.OnCollisionCheckWeapon = newOnCollisionCheckWeapon   
     
      -- Marks UnitBeingBuilt's weapons as friendly
      neutralizeWeapons(unitBeingBuilt)
   
     -- Modify build function to pass immunity on to other units
      if unitBeingBuilt.OnStartBuild and unitBeingBuilt.OldOnStartBuild == nil then
         unitBeingBuilt.OldOnStartBuild = unitBeingBuilt.OnStartBuild
         unitBeingBuilt.OnStartBuild = newOnStartBuild
      end
   else
      -- do nothing
   end
end

## Replacement Collision Check function
newOnCollisionCheck = function(self, other, firingWeapon)
   if other.friendly == 1 then
      return false
   else
      return self.OldOnCollisionCheck(self, other, firingWeapon)   
   end
end

## Replacement function for Projectile Creation
newCreateProjectileForWeapon = function(self, muzzle)
    local proj = self.OldCreateProjectileForWeapon(self, muzzle)
    proj.friendly = 1
end

## Replacement Collision Check function used by Beam Weapons
newOnCollisionCheckWeapon = function(self, firingWeapon)
   if firingWeapon.friendly == 1 then
      return false
   else
      return self.OldOnCollisionCheckWeapon(self, firingWeapon)   
   end
end

## Mark weapons as friendly
neutralizeWeapons = function(unit)
  for i = 1, unit:GetWeaponCount() do
      local weapon = unit:GetWeapon(i)
      local bp = unit:GetBlueprint().Weapon[i]
      if bp.ProjectileId != "" then
          weapon.OldCreateProjectileForWeapon = weapon.CreateProjectileForWeapon
          weapon.CreateProjectileForWeapon = newCreateProjectileForWeapon
      else
         weapon.friendly = 1
      end
   end
end